#!/bin/bash
############################################################
# source: 
shell_interpreter_fix_testing.sh
# author: devenkong(18151155@qq.com)
# date: 2021-10-02
############################################################
# Copyright (C) 2022- Free Software Foundation, Inc.
# This configure script is free software; the Free Software 
# Foundation gives unlimited permission to copy, distribute 
# and modify it.
############################################################
# note:
#     测试脚本解释器是修正程序fix-shebang。
############################################################


TESTING_ROOT_DIR=
TESTING_TMP_DIR=~/.testing/$0/


main ()
{
	local name=
	local path=
	
	echo "\n1.无参数。对文件中原有的interpreter进行路径修正，如果不存在，输出提示信息。"
	rm $TESTING_TMP_DIR/ -rf
	mkdir -p $TESTING_TMP_DIR
	cp 1.* *.sh scripts $TESTING_TMP_DIR -rf
	cd $TESTING_TMP_DIR
	chmod +x *.sh scripts/*
	
	echo -ne "[shebang]:" > a.txt
	head -n 1 scripts/1.interpreter_sh_with_wrong_path.sh > a.txt
	
	which bash
	which sh
	fix-shebang scripts/1.interpreter_sh_with_wrong_path.sh
	
	echo -ne "[shebang]:" > b.txt
	head -n 1 scripts/1.interpreter_sh_with_wrong_path.sh > b.txt
	
	if [[ -n `diff a.txt b.txt` ]]; then
		echo "interpreter fixed."
	else
		echo "interpreter not fixed."
	fi
	./scripts/1.interpreter_sh_with_wrong_path.sh
	[[ $? != 0 ]] && echo "interpreter not fixed"
	
	cd - > /dev/null
	
	exit
	
	echo "\n2.-c参数。脚本自带的interpreter无效时，使用默认interpreter。"
	rm $TESTING_TMP_DIR/ -rf
	mkdir -p $TESTING_TMP_DIR
	cp 1.* *.sh scripts $TESTING_TMP_DIR -rf
	cd $TESTING_TMP_DIR
	chmod +x *.sh scripts/*
	
	echo -ne "[shebang]:" > a.txt
	head -n 1 scripts/1.interpreter_abc_not_valid_in_system.sh > a.txt
	
	fix-shebang -c scripts/*
	
	echo -ne "[shebang]:" > b.txt
	head -n 1 scripts/1.interpreter_abc_not_valid_in_system.sh > b.txt
	
	if [[ -n `diff a.txt b.txt` ]]; then
		echo "interpreter fixed."
	else
		echo "interpreter not fixed."
	fi
	./scripts/1.interpreter_abc_not_valid_in_system.sh
	[[ $? != 0 ]] && echo "interpreter not fixed"
	
	cd - > /dev/null
	
	
	echo "\n3.-i参数。不对文件进行处理，只输出信息。"
	rm $TESTING_TMP_DIR/ -rf
	mkdir -p $TESTING_TMP_DIR
	cp 1.* *.sh scripts $TESTING_TMP_DIR -rf
	cd $TESTING_TMP_DIR
	chmod +x *.sh scripts/*
	
	echo -ne "[shebang]:" > a.txt
	head -n 1 scripts/{1.interpreter_sh_with_wrong_path.sh, 1.interpreter_abc_not_valid_in_system.sh} > a.txt
	
	fix-shebang -i scripts/*
	
	echo -ne "[shebang]:" > b.txt
	head -n 1 scripts/{1.interpreter_sh_with_wrong_path.sh, 1.interpreter_abc_not_valid_in_system.sh} > b.txt
	
	if [[ -n `diff a.txt b.txt` ]]; then
		echo "interpreter fixed."
	else
		echo "interpreter not fixed."
	fi
	./scripts/1.interpreter_sh_with_wrong_path.sh > /dev/null
	[[ $? != 0 ]] && echo "interpreter not fixed"
	./scripts/1.interpreter_abc_not_valid_in_system.sh > /dev/null
	[[ $? != 0 ]] && echo "interpreter not fixed"
	
	cd - > /dev/null
	
	
	echo "\n4.-i -r参数。读取脚本文件1.interpreter_sh_with_wrong_path.sh中的解释器路径和名称，并输出。"
	rm $TESTING_TMP_DIR/ -rf
	mkdir -p $TESTING_TMP_DIR
	cp 1.* *.sh scripts $TESTING_TMP_DIR -rf
	cd $TESTING_TMP_DIR
	chmod +x *.sh scripts/*
	
	echo -ne "[shebang]:" > a.txt
	head -n 1 scripts/{1.interpreter_sh_with_wrong_path.sh, 1.interpreter_bash_with_wrong_path.sh} > a.txt
	
	fix-shebang -i -r scripts/*
	
	echo -ne "[shebang]:" > b.txt
	head -n 1 scripts/{1.interpreter_sh_with_wrong_path.sh, 1.interpreter_bash_with_wrong_path.sh} > b.txt
	
	if [[ -n `diff a.txt b.txt` ]]; then
		echo "interpreter fixed."
	else
		echo "interpreter not fixed."
	fi
	
	cd - > /dev/null
	
	
	echo "\n5.-i -v参数。只输出待处理文件的文件名。"
	rm $TESTING_TMP_DIR/ -rf
	mkdir -p $TESTING_TMP_DIR
	cp 1.* *.sh scripts $TESTING_TMP_DIR -rf
	cd $TESTING_TMP_DIR
	chmod +x *.sh scripts/*
	
	echo -ne "[shebang]:" > a.txt
	head -n 1 scripts/{1.interpreter_sh_with_wrong_path.sh, 1.interpreter_bash_with_wrong_path.sh} > a.txt
	
	fix-shebang -i -v scripts/*
	
	echo -ne "[shebang]:" > b.txt
	head -n 1 scripts/{1.interpreter_sh_with_wrong_path.sh, 1.interpreter_bash_with_wrong_path.sh} > b.txt
	
	if [[ -n `diff a.txt b.txt` ]]; then
		echo "interpreter fixed."
	else
		echo "interpreter not fixed."
	fi
	
	cd - > /dev/null
	
	
	echo "\n6.-t参数。只对指定扩展名进行处理。"
	rm $TESTING_TMP_DIR/ -rf
	mkdir -p $TESTING_TMP_DIR
	cp 1.* *.sh scripts $TESTING_TMP_DIR -rf
	cd $TESTING_TMP_DIR
	chmod +x *.sh scripts/*
	
	echo -ne "[shebang]:" > a.txt
	head -n 1 scripts/* > a.txt
	
	fix-shebang -t .bzsh -c scripts/*
	
	echo -ne "[shebang]:" > b.txt
	head -n 1 scripts/* > b.txt
	
	if [[ -n `diff a.txt b.txt` ]]; then
		echo "interpreter fixed."
	else
		echo "interpreter not fixed."
	fi
	./scripts/1.interpreter_bzsh_with_extname_bzsh_for_filter.sh
	[[ $? != 0 ]] && echo "interpreter not fixed"，
	
	cd - > /dev/null
	
	
	echo "\n7.-d参数。对指定interpreter进行处理。" 
	rm $TESTING_TMP_DIR/ -rf
	mkdir -p $TESTING_TMP_DIR
	cp 1.* *.sh scripts $TESTING_TMP_DIR -rf
	cd $TESTING_TMP_DIR
	chmod +x *.sh scripts/*
	
	echo -ne "[shebang]:" > a.txt
	head -n 1 scripts/1.interpreter_xyz_used_for_replace_testing.sh > a.txt
	
	fix-shebang -d xyz scripts/1.interpreter_xyz_used_for_replace_testing.sh
	
	echo -ne "[shebang]:" > b.txt
	head -n 1 scripts/1.interpreter_xyz_used_for_replace_testing.sh > b.txt
	
	if [[ -n `diff a.txt b.txt` ]]; then
		echo "interpreter fixed."
	else
		echo "interpreter not fixed."
	fi
	./scripts/1.interpreter_xyz_used_for_replace_testing.sh
	[[ $? != 0 ]] && echo "interpreter not fixed"
	
	cd - > /dev/null
	
	
	echo "\n8.-s参数。设定一个默认interpreter。"
	rm $TESTING_TMP_DIR/ -rf
	mkdir -p $TESTING_TMP_DIR
	cp 1.* *.sh scripts $TESTING_TMP_DIR -rf
	cd $TESTING_TMP_DIR
	chmod +x *.sh scripts/*
	
	echo -ne "[shebang]:" > a.txt
	head -n 1 scripts/{1.interpreter_sh_with_wrong_path.sh, 1.interpreter_bash_with_wrong_path.sh} > a.txt
	
	fix-shebang -d bash -s sh scripts/{1.interpreter_sh_with_wrong_path.sh, 1.interpreter_bash_with_wrong_path.sh}
	
	echo -ne "[shebang]:" > b.txt
	head -n 1 scripts/{1.interpreter_sh_with_wrong_path.sh, 1.interpreter_bash_with_wrong_path.sh} > b.txt
	
	if [[ -n `diff a.txt b.txt` ]]; then
		echo "interpreter fixed."
	else
		echo "interpreter not fixed."
	fi
	./scripts/1.interpreter_sh_with_wrong_path.sh > /dev/null
	[[ $? != 0 ]] && echo "interpreter not fixed"
	./scripts/1.interpreter_bash_with_wrong_path.sh
	[[ $? != 0 ]] && echo "interpreter not fixed"
	
	cd - > /dev/null
	
	
	echo "\n9.-f参数。对所有待处理文件强制使用默认或-s设定的interpreter。"
	rm $TESTING_TMP_DIR/ -rf
	mkdir -p $TESTING_TMP_DIR
	cp 1.* *.sh scripts $TESTING_TMP_DIR -rf
	cd $TESTING_TMP_DIR
	chmod +x *.sh scripts/*
	
	echo -ne "[shebang]:" > a.txt
	head -n 1 scripts/1.interpreter_sh_with_wrong_path.sh > a.txt
	
	fix-shebang -f scripts/*
	
	echo -ne "[shebang]:" > b.txt
	head -n 1 scripts/1.interpreter_sh_with_wrong_path.sh > b.txt
	
	if [[ -n `diff a.txt b.txt` ]]; then
		echo "interpreter fixed."
	else
		echo "interpreter not fixed."
	fi
	./scripts/1.interpreter_sh_with_wrong_path.sh > /dev/null
	[[ $? != 0 ]] && echo "1.interpreter_sh_with_wrong_path.sh interpreter not fixed"
	./scripts/1.interpreter_bash_with_wrong_path.sh > /dev/null
	[[ $? != 0 ]] && echo "1.interpreter_sh_with_wrong_path.sh interpreter not fixed"
	./scripts/1.interpreter_xyz_used_for_replace_testing.sh
	[[ $? != 0 ]] && echo "1.interpreter_sh_with_wrong_path.sh interpreter not fixed"
	
	cd - > /dev/null
	
	
	echo "\n10.解释器带参数。"
	rm $TESTING_TMP_DIR/ -rf
	mkdir -p $TESTING_TMP_DIR
	cp 1.* *.sh scripts $TESTING_TMP_DIR -rf
	cd $TESTING_TMP_DIR
	chmod +x *.sh scripts/*
	
	echo -ne "[shebang]:" > a.txt
	head -n 1 scripts/1.interpreter_sh_with_parameter.sh > a.txt
	
	fix-shebang -f scripts/*
	
	echo -ne "[shebang]:" > b.txt
	head -n 1 scripts/1.interpreter_sh_with_parameter.sh > b.txt
	
	if [[ -n `diff a.txt b.txt` ]]; then
		echo "interpreter fixed."
	else
		echo "interpreter not fixed."
	fi
	./scripts/1.interpreter_sh_with_parameter.sh > /dev/null
	[[ $? != 0 ]] && echo "1.interpreter_sh_with_parameter.sh interpreter not fixed"
	
	cd - > /dev/null
	
	echo "\n11.解释器带参数。"
	rm $TESTING_TMP_DIR/ -rf
	mkdir -p $TESTING_TMP_DIR
	cp 1.* *.sh scripts $TESTING_TMP_DIR -rf
	cd $TESTING_TMP_DIR
	chmod +x *.sh scripts/*
	
	echo -ne "[shebang]:" > a.txt
	head -n 1 scripts/1.interpreter_sh_with_parameter.sh > a.txt
	
	fix-shebang -f scripts/*
	
	echo -ne "[shebang]:" > b.txt
	head -n 1 scripts/1.interpreter_sh_with_parameter.sh > b.txt
	
	if [[ -n `diff a.txt b.txt` ]]; then
		echo "interpreter fixed."
	else
		echo "interpreter not fixed."
	fi
	./scripts/1.interpreter_sh_with_parameter.sh > /dev/null
	[[ $? != 0 ]] && echo "1.interpreter_sh_with_parameter.sh interpreter not fixed"
	
	cd - > /dev/null
	
	
	echo "\n12.参数组合。find * | xargs fix-shebang -r -t .sh -d sh -s bash -v -f"
	rm $TESTING_TMP_DIR/ -rf
	mkdir -p $TESTING_TMP_DIR
	cp 1.* *.sh scripts $TESTING_TMP_DIR -rf
	cd $TESTING_TMP_DIR
	chmod +x *.sh scripts/*
	
	echo -ne "[shebang]:" > a.txt
	head -n 1 scripts/{1.interpreter_sh_with_wrong_path.sh, 1.interpreter_bash_with_wrong_path.sh} > a.txt
	
	fix-shebang -r -t .sh -d sh -s bash -v -f scripts/*
	
	echo -ne "[shebang]:" > b.txt
	head -n 1 scripts/{1.interpreter_sh_with_wrong_path.sh, 1.interpreter_bash_with_wrong_path.sh} > b.txt
	
	if [[ -n `diff a.txt b.txt` ]]; then
		echo "interpreter fixed."
	else
		echo "interpreter not fixed."
	fi
	./scripts/1.interpreter_sh_with_wrong_path.sh
	./scripts/1.interpreter_bash_with_wrong_path.sh
	
	cd - > /dev/null
	
}


main
